/*
 * Level.h
 *
 * Copyright (C) 2009 Matthias Schiffer <matthias@gamezock.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ZOOM_LEVEL_H_
#define ZOOM_LEVEL_H_

#include "TriangleRecord.h"

#include <string>
#include <map>
#include <list>

#include <boost/shared_ptr.hpp>
#include <libxml/parser.h>
#include <vmmlib/vector.hpp>

namespace Zoom {

class Level {
  public:
    static boost::shared_ptr<Level> loadLevel(const std::string &filename);

    struct WallData : public TriangleRecord::TriangleData {
      std::string texture;
    };

    struct Room {
      std::string id;
      std::list<TriangleRecord> walls;
    };

    const std::list<Room> &getRooms() const {
      return rooms;
    }

  private:
    std::list<Room> rooms;
    std::map<std::string, unsigned> textures;

    std::string name, description;
    vmml::vec3f startPoint;

    Level() {}

    void loadLevelInfo(xmlNodePtr infoNode);
    void loadRooms(xmlNodePtr roomsNode);
    void loadRoom(xmlNodePtr roomNode);
    void loadTextures(xmlNodePtr texturesNode);

    static TriangleRecord loadWall(xmlNodePtr wallNode);
    static vmml::vec3f loadVector(xmlNodePtr node);

    static bool validateLevel(xmlDocPtr doc);
};

}

#endif /* ZOOM_LEVEL_H_ */
