/*
 * MathUtil.cpp
 *
 * Copyright (C) 2009 Matthias Schiffer <matthias@gamezock.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "MathUtil.h"
#include <cmath>

namespace Zoom {

const float MathUtil::EPSILON = 1E-6;

vmml::vec3f MathUtil::Plane::intersection(const Ray &ray) const {
  float r = (d - ray.getVertex().dot(normal))/ray.getDirection().dot(normal);

  return ray.getVertex() + r*ray.getDirection();
}


vmml::mat4f MathUtil::perspective(float fovy, float aspect, float zNear) {
  float f = 1/std::tan(fovy*M_PI/360);

  vmml::mat4f ret(vmml::mat4f::ZERO);
  ret[0][0] = f/aspect;
  ret[1][1] = f;
  ret[2][2] = EPSILON-1;
  ret[2][3] = (EPSILON-2)*zNear;
  ret[3][2] = -1;

  return ret;
}

}
